@@ -44,7 +44,7 @@
 // abstract Iterator template						/*{{{*/
 /* This template provides the very basic iterator methods we
    need to have for doing some walk-over-the-cache magic */
-template<typename Str, typename Itr> class pkgCache::Iterator :
+template<typename Str, typename Itr> class APT_PUBLIC pkgCache::Iterator :
 			public std::iterator<std::forward_iterator_tag, Str> {
 	/** \brief Returns the Pointer for this struct in the owner
 	 *  The implementation of this method should be pretty short
@@ -80,6 +80,7 @@ template<typename Str, typename Itr> class pkgCache::Iterator :
 	// Mixed stuff
 	inline bool IsGood() const { return S && Owner && ! end();}
 	inline unsigned long Index() const {return S - OwnerPointer();}
+	inline map_pointer<Str> MapPointer() const {return map_pointer<Str>(Index()) ;}
 
 	void ReMap(void const * const oldMap, void const * const newMap) {
 		if (Owner == 0 || S == 0)
@@ -97,7 +98,7 @@ template<typename Str, typename Itr> class pkgCache::Iterator :
    interest in package names can iterate easily over the names, so the
    different architectures can be treated as of the "same" package
    (apt internally treat them as totally different packages) */
-class pkgCache::GrpIterator: public Iterator<Group, GrpIterator> {
+class APT_PUBLIC pkgCache::GrpIterator: public Iterator<Group, GrpIterator> {
 	long HashIndex;
 
 	public:
@@ -116,6 +117,7 @@ class pkgCache::GrpIterator: public Iterator<Group, GrpIterator> {
 
 	inline const char *Name() const {return S->Name == 0?0:Owner->StrP + S->Name;}
 	inline PkgIterator PackageList() const;
+	inline VerIterator VersionsInSource() const;
 	PkgIterator FindPkg(APT::StringView Arch = APT::StringView("any", 3)) const;
 	/** \brief find the package with the "best" architecture
 
@@ -137,7 +139,7 @@ class pkgCache::GrpIterator: public Iterator<Group, GrpIterator> {
 };
 									/*}}}*/
 // Package Iterator							/*{{{*/
-class pkgCache::PkgIterator: public Iterator<Package, PkgIterator> {
+class APT_PUBLIC pkgCache::PkgIterator: public Iterator<Package, PkgIterator> {
 	long HashIndex;
 
 	public:
@@ -183,7 +185,7 @@ class pkgCache::PkgIterator: public Iterator<Package, PkgIterator> {
 };
 									/*}}}*/
 // Version Iterator							/*{{{*/
-class pkgCache::VerIterator : public Iterator<Version, VerIterator> {
+class APT_PUBLIC pkgCache::VerIterator : public Iterator<Version, VerIterator> {
 	public:
 	inline Version* OwnerPointer() const {
 		return (Owner != 0) ? Owner->VerP : 0;
@@ -193,6 +195,13 @@ class pkgCache::VerIterator : public Iterator<Version, VerIterator> {
 	inline VerIterator& operator++() {if (S != Owner->VerP) S = Owner->VerP + S->NextVer; return *this;}
 	inline VerIterator operator++(int) { VerIterator const tmp(*this); operator++(); return tmp; }
 
+	inline VerIterator NextInSource()
+	{
+	   if (S != Owner->VerP)
+	      S = Owner->VerP + S->NextInSource;
+	   return *this;
+	}
+
 	// Comparison
 	int CompareVer(const VerIterator &B) const;
 	/** \brief compares two version and returns if they are similar
@@ -233,6 +242,20 @@ class pkgCache::VerIterator : public Iterator<Version, VerIterator> {
 	bool Automatic() const;
 	VerFileIterator NewestFile() const;
 
+#ifdef APT_COMPILING_APT
+	inline unsigned int PhasedUpdatePercentage() const
+	{
+	   return (static_cast<Version::Extra *>(Owner->Map.Data()) + S->d)->PhasedUpdatePercentage;
+	}
+	inline bool PhasedUpdatePercentage(unsigned int percentage)
+	{
+	   if (percentage > 100)
+	      return false;
+	   (static_cast<Version::Extra *>(Owner->Map.Data()) + S->d)->PhasedUpdatePercentage = static_cast<uint8_t>(percentage);
+	   return true;
+	}
+#endif
+
 	inline VerIterator(pkgCache &Owner,Version *Trg = 0) : Iterator<Version, VerIterator>(Owner, Trg) {
 		if (S == 0)
 			S = OwnerPointer();
@@ -241,7 +264,7 @@ class pkgCache::VerIterator : public Iterator<Version, VerIterator> {
 };
 									/*}}}*/
 // Description Iterator							/*{{{*/
-class pkgCache::DescIterator : public Iterator<Description, DescIterator> {
+class APT_PUBLIC pkgCache::DescIterator : public Iterator<Description, DescIterator> {
 	public:
 	inline Description* OwnerPointer() const {
 		return (Owner != 0) ? Owner->DescP : 0;
@@ -267,7 +290,7 @@ class pkgCache::DescIterator : public Iterator<Description, DescIterator> {
 };
 									/*}}}*/
 // Dependency iterator							/*{{{*/
-class pkgCache::DepIterator : public Iterator<Dependency, DepIterator> {
+class APT_PUBLIC pkgCache::DepIterator : public Iterator<Dependency, DepIterator> {
 	enum {DepVer, DepRev} Type;
 	DependencyData * S2;
 
@@ -285,7 +308,7 @@ class pkgCache::DepIterator : public Iterator<Dependency, DepIterator> {
 	inline PkgIterator TargetPkg() const {return PkgIterator(*Owner,Owner->PkgP + S2->Package);}
 	inline PkgIterator SmartTargetPkg() const {PkgIterator R(*Owner,0);SmartTargetPkg(R);return R;}
 	inline VerIterator ParentVer() const {return VerIterator(*Owner,Owner->VerP + S->ParentVer);}
-	inline PkgIterator ParentPkg() const {return PkgIterator(*Owner,Owner->PkgP + Owner->VerP[S->ParentVer].ParentPkg);}
+	inline PkgIterator ParentPkg() const {return PkgIterator(*Owner,Owner->PkgP + Owner->VerP[uint32_t(S->ParentVer)].ParentPkg);}
 	inline bool Reverse() const {return Type == DepRev;}
 	bool IsCritical() const APT_PURE;
 	bool IsNegative() const APT_PURE;
@@ -313,15 +336,15 @@ class pkgCache::DepIterator : public Iterator<Dependency, DepIterator> {
 	struct DependencyProxy
 	{
 	   map_stringitem_t &Version;
-	   map_pointer_t &Package;
+	   map_pointer<pkgCache::Package> &Package;
 	   map_id_t &ID;
 	   unsigned char &Type;
 	   unsigned char &CompareOp;
-	   map_pointer_t &ParentVer;
-	   map_pointer_t &DependencyData;
-	   map_pointer_t &NextRevDepends;
-	   map_pointer_t &NextDepends;
-	   map_pointer_t &NextData;
+	   map_pointer<pkgCache::Version> &ParentVer;
+	   map_pointer<pkgCache::DependencyData> &DependencyData;
+	   map_pointer<Dependency> &NextRevDepends;
+	   map_pointer<Dependency> &NextDepends;
+	   map_pointer<pkgCache::DependencyData> &NextData;
 	   DependencyProxy const * operator->() const { return this; }
 	   DependencyProxy * operator->() { return this; }
 	};
@@ -352,7 +375,7 @@ class pkgCache::DepIterator : public Iterator<Dependency, DepIterator> {
 };
 									/*}}}*/
 // Provides iterator							/*{{{*/
-class pkgCache::PrvIterator : public Iterator<Provides, PrvIterator> {
+class APT_PUBLIC pkgCache::PrvIterator : public Iterator<Provides, PrvIterator> {
 	enum {PrvVer, PrvPkg} Type;
 
 	public:
@@ -370,7 +393,7 @@ class pkgCache::PrvIterator : public Iterator<Provides, PrvIterator> {
 	inline const char *ProvideVersion() const {return S->ProvideVersion == 0?0:Owner->StrP + S->ProvideVersion;}
 	inline PkgIterator ParentPkg() const {return PkgIterator(*Owner,Owner->PkgP + S->ParentPkg);}
 	inline VerIterator OwnerVer() const {return VerIterator(*Owner,Owner->VerP + S->Version);}
-	inline PkgIterator OwnerPkg() const {return PkgIterator(*Owner,Owner->PkgP + Owner->VerP[S->Version].ParentPkg);}
+	inline PkgIterator OwnerPkg() const {return PkgIterator(*Owner,Owner->PkgP + Owner->VerP[uint32_t(S->Version)].ParentPkg);}
 
 	/* MultiArch can be translated to SingleArch for an resolver and we did so,
 	   by adding provides to help the resolver understand the problem, but
@@ -393,7 +416,7 @@ class pkgCache::PrvIterator : public Iterator<Provides, PrvIterator> {
 };
 									/*}}}*/
 // Release file								/*{{{*/
-class pkgCache::RlsFileIterator : public Iterator<ReleaseFile, RlsFileIterator> {
+class APT_PUBLIC pkgCache::RlsFileIterator : public Iterator<ReleaseFile, RlsFileIterator> {
 	public:
 	inline ReleaseFile* OwnerPointer() const {
 		return (Owner != 0) ? Owner->RlsFileP : 0;
@@ -422,7 +445,7 @@ class pkgCache::RlsFileIterator : public Iterator<ReleaseFile, RlsFileIterator>
 };
 									/*}}}*/
 // Package file								/*{{{*/
-class pkgCache::PkgFileIterator : public Iterator<PackageFile, PkgFileIterator> {
+class APT_PUBLIC pkgCache::PkgFileIterator : public Iterator<PackageFile, PkgFileIterator> {
 	public:
 	inline PackageFile* OwnerPointer() const {
 		return (Owner != 0) ? Owner->PkgFileP : 0;
@@ -456,7 +479,7 @@ class pkgCache::PkgFileIterator : public Iterator<PackageFile, PkgFileIterator>
 };
 									/*}}}*/
 // Version File								/*{{{*/
-class pkgCache::VerFileIterator : public pkgCache::Iterator<VerFile, VerFileIterator> {
+class APT_PUBLIC pkgCache::VerFileIterator : public pkgCache::Iterator<VerFile, VerFileIterator> {
 	public:
 	inline VerFile* OwnerPointer() const {
 		return (Owner != 0) ? Owner->VerFileP : 0;
@@ -467,14 +490,14 @@ class pkgCache::VerFileIterator : public pkgCache::Iterator<VerFile, VerFileIter
 	inline VerFileIterator operator++(int) { VerFileIterator const tmp(*this); operator++(); return tmp; }
 
 	// Accessors
-	inline PkgFileIterator File() const {return PkgFileIterator(*Owner,S->File + Owner->PkgFileP);}
+	inline PkgFileIterator File() const {return PkgFileIterator(*Owner, Owner->PkgFileP + S->File);}
 
 	inline VerFileIterator() : Iterator<VerFile, VerFileIterator>() {}
 	inline VerFileIterator(pkgCache &Owner,VerFile *Trg) : Iterator<VerFile, VerFileIterator>(Owner, Trg) {}
 };
 									/*}}}*/
 // Description File							/*{{{*/
-class pkgCache::DescFileIterator : public Iterator<DescFile, DescFileIterator> {
+class APT_PUBLIC pkgCache::DescFileIterator : public Iterator<DescFile, DescFileIterator> {
 	public:
 	inline DescFile* OwnerPointer() const {
 		return (Owner != 0) ? Owner->DescFileP : 0;
@@ -485,7 +508,7 @@ class pkgCache::DescFileIterator : public Iterator<DescFile, DescFileIterator> {
 	inline DescFileIterator operator++(int) { DescFileIterator const tmp(*this); operator++(); return tmp; }
 
 	// Accessors
-	inline PkgFileIterator File() const {return PkgFileIterator(*Owner,S->File + Owner->PkgFileP);}
+	inline PkgFileIterator File() const {return PkgFileIterator(*Owner, Owner->PkgFileP + S->File);}
 
 	inline DescFileIterator() : Iterator<DescFile, DescFileIterator>() {}
 	inline DescFileIterator(pkgCache &Owner,DescFile *Trg) : Iterator<DescFile, DescFileIterator>(Owner, Trg) {}
@@ -494,6 +517,10 @@ class pkgCache::DescFileIterator : public Iterator<DescFile, DescFileIterator> {
 // Inlined Begin functions can't be in the class because of order problems /*{{{*/
 inline pkgCache::PkgIterator pkgCache::GrpIterator::PackageList() const
        {return PkgIterator(*Owner,Owner->PkgP + S->FirstPackage);}
+       inline pkgCache::VerIterator pkgCache::GrpIterator::VersionsInSource() const
+       {
+	  return VerIterator(*Owner, Owner->VerP + S->VersionsInSource);
+       }
 inline pkgCache::VerIterator pkgCache::PkgIterator::VersionList() const
        {return VerIterator(*Owner,Owner->VerP + S->VersionList);}
 inline pkgCache::VerIterator pkgCache::PkgIterator::CurrentVer() const
