@@ -86,7 +86,7 @@ bool pkgCacheGenerator::Start()
       Map.UsePools(*Cache.HeaderP->Pools,sizeof(Cache.HeaderP->Pools)/sizeof(Cache.HeaderP->Pools[0]));
 
       // Starting header
-      new (Cache.HeaderP) pkgCache::Header();
+      *Cache.HeaderP = pkgCache::Header();
 
       // make room for the hashtables for packages and groups
       if (Map.RawAllocate(2 * (Cache.HeaderP->GetHashTableSize() * sizeof(map_pointer<void>))) == 0)
@@ -873,6 +873,10 @@ map_pointer<pkgCache::Version> pkgCacheGenerator::NewVersion(pkgCache::VerIterat
    
    // Fill it in
    Ver = pkgCache::VerIterator(Cache,Cache.VerP + Version);
+   Ver->d = AllocateInMap<pkgCache::Version::Extra>();
+   if (not Ver.PhasedUpdatePercentage(100))
+      abort();
+
    //Dynamic<pkgCache::VerIterator> DynV(Ver); // caller MergeListVersion already takes care of it
    Ver->NextVer = Next;
    Ver->ParentPkg = ParentPkg;
